package JointFeldmanProtocol;

import FeldmanVerifiableSecretSharing.ShamirSecretSharing.Polynomial;
import FeldmanVerifiableSecretSharing.VerifiableSecretSharing;
import org.bouncycastle.util.Arrays;
import org.factcenter.qilin.primitives.concrete.Zpstar;

import java.math.BigInteger;
import java.util.Random;
import java.util.Set;

/**
 * Created by Tzlil on 2/5/2016.
 *
 * an implementation of a version of Pedersen's distributed key generation protocol
 */
public class DKG extends VerifiableSecretSharing{

    private Network.User user;
    private Polynomial.Point[][] shares;
    private BigInteger[][] commitmentsArray;
    private Set<Integer> QUAL;

    private BigInteger x;
    private BigInteger y;
    private BigInteger[] commitments;

    public DKG(int t, int n, BigInteger x, Random random, BigInteger p, BigInteger q, BigInteger g) {
        super(t, n, x, random, p, q, g);
    }

    private void stage1(){
        int n = getN();
        int i = user.getID();
        BigInteger[] commitments = super.getCommitments();
        System.arraycopy(commitments, 0, commitmentsArray[i - 1], 0, commitmentsArray[i - 1].length);

        Network.Message message = null;
        for (int j = 1; j <= commitmentsArray[i - 1].length; j ++){
            //message = new Message(Type.Commitment, Shares[i - 1][j - 1])
            user.sendBroadcast(message);
        }


        for (int j = 1; j <= shares[i - 1].length; j ++){
            shares[i - 1][j - 1] = getShare(j);
        }
        for (int j = 1; j <= n ; j++ ){
            if(j != i){
                //message = new Message(Type.Share, Shares[i - 1][j - 1])
                user.send(j,message);
            }
        }
        //Todo receive messages
    }

    private void stage2(){
        int n = getN();
        BigInteger g = getGenerator();
        Zpstar zpstar = getZpstar();
        int i = user.getID();
        Network.Message message = null;
        for (int j = 1; j <= n ; j++ ){
            if(zpstar.multiply(g,shares[i - 1][j - 1].y).equals(verify(j,commitmentsArray[j],zpstar))){
                QUAL.add(j);
            }else{
                //message = new Message(Type.Complaint, j)
                user.sendBroadcast(message);
            }
        }

    }

    private void stage3(){

        //Todo receive something private from each complaint + send what necessary


    }

    private void stage4(){
        Network.Message message = null;
        // message = new Message(Type.Y, super.getY())
        user.sendBroadcast(message);
        Zpstar zpstar = getZpstar();
        BigInteger y = zpstar.zero();
        for (Network.User user:this.user.getNetwork()) {
            //Todo receive yi from all i in QUAL and calc y total
        }
        int t = getT();
        this.commitments = new BigInteger[t];
        BigInteger commitment;
        for (int k = 1; k <= t ; k++){
            commitment = zpstar.zero();
            for (int i : QUAL) {
                commitment = zpstar.add(commitment,commitmentsArray[i - 1][k - 1]);
            }
            commitments[k - 1] = commitment;
        }

        int j = user.getID();
        BigInteger x = BigInteger.ZERO;
        for (int i : QUAL) {
            x = x.add(shares[i][j].y);
        }
        this.x = x.mod(getQ());
    }

    @Override
    public BigInteger getY() {
        return y;
    }

    @Override
    public BigInteger[] getCommitments() {
        return Arrays.clone(commitments);
    }

    @Override
    protected Polynomial.Point getShare(int i) {
        return null;
    }
}
